@@ -259,23 +259,46 @@ public:
 	}
 	Effect *add_effect(Effect *effect, const std::vector<Effect *> &inputs);
 
-	// Adds an RGBA output. Note that you can have at most one RGBA output and one
-	// Y'CbCr output (see below for details).
+	// Adds an RGBA output. Note that you can have at most one RGBA output and two
+	// Y'CbCr outputs (see below for details).
 	void add_output(const ImageFormat &format, OutputAlphaFormat alpha_format);
 
-	// Adds an YCbCr output. Note that you can only have one output.
-	// Currently, only chunked packed output is supported, and only 4:4:4
-	// (so chroma_subsampling_x and chroma_subsampling_y must both be 1).
+	// Adds an YCbCr output. Note that you can only have at most two Y'CbCr
+	// outputs, and they must have the same <ycbcr_format> and <type>.
+	// (This limitation may be lifted in the future, to allow e.g. simultaneous
+	// 8- and 10-bit output. Currently, multiple Y'CbCr outputs are only
+	// useful in some very limited circumstances, like if one texture goes
+	// to some place you cannot easily read from later.)
 	//
-	// If you have both RGBA and Y'CbCr output, the RGBA output will come
+	// Only 4:4:4 output is supported due to fragment shader limitations,
+	// so chroma_subsampling_x and chroma_subsampling_y must both be 1.
+	// <type> should match the data type of the FBO you are rendering to,
+	// so that if you use 16-bit output (GL_UNSIGNED_SHORT), you will get
+	// 8-, 10- or 12-bit output correctly as determined by <ycbcr_format.num_levels>.
+	// Using e.g. ycbcr_format.num_levels == 1024 with GL_UNSIGNED_BYTE is
+	// nonsensical and invokes undefined behavior.
+	//
+	// If you have both RGBA and Y'CbCr output(s), the RGBA output will come
 	// in the last draw buffer. Also, <format> and <alpha_format> must be
 	// identical between the two.
 	void add_ycbcr_output(const ImageFormat &format, OutputAlphaFormat alpha_format,
 	                      const YCbCrFormat &ycbcr_format,
-			      YCbCrOutputSplitting output_splitting = YCBCR_OUTPUT_INTERLEAVED);
+			      YCbCrOutputSplitting output_splitting = YCBCR_OUTPUT_INTERLEAVED,
+	                      GLenum output_type = GL_UNSIGNED_BYTE);
+
+	// Change Y'CbCr output format. (This can be done also after finalize()).
+	// Note that you are not allowed to change subsampling parameters;
+	// however, you can change the color space parameters, ie.,
+	// luma_coefficients, full_range and num_levels.
+	void change_ycbcr_output_format(const YCbCrFormat &ycbcr_format);
 
 	// Set number of output bits, to scale the dither.
 	// 8 is the right value for most outputs.
+	//
+	// Special note for 10- and 12-bit Y'CbCr packed into GL_UNSIGNED_SHORT:
+	// This is relative to the actual output, not the logical one, so you should
+	// specify 16 here, not 10 or 12.
+	//
 	// The default, 0, is a special value that means no dither.
 	void set_dither_bits(unsigned num_bits)
 	{
@@ -479,13 +502,16 @@ private:
 	ImageFormat output_format;
 	OutputAlphaFormat output_alpha_format;
 
-	bool output_color_rgba, output_color_ycbcr;
-	YCbCrFormat output_ycbcr_format;              // If output_color_ycbcr is true.
-	YCbCrOutputSplitting output_ycbcr_splitting;  // If output_color_ycbcr is true.
+	bool output_color_rgba;
+	int num_output_color_ycbcr;                      // Max 2.
+	YCbCrFormat output_ycbcr_format;                 // If num_output_color_ycbcr is > 0.
+	GLenum output_ycbcr_type;                        // If num_output_color_ycbcr is > 0.
+	YCbCrOutputSplitting output_ycbcr_splitting[2];  // If num_output_color_ycbcr is > N.
 
 	std::vector<Node *> nodes;
 	std::map<Effect *, Node *> node_map;
 	Effect *dither_effect;
+	Node *ycbcr_conversion_effect_node;
 
 	std::vector<Input *> inputs;  // Also contained in nodes.
 	std::vector<Phase *> phases;
